/*******************************************************************************
This file is part of mdictionary.

mdictionary is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

mdictionary is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License 
along with mdictionary; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Copyright 2006 ComArch S.A.
*******************************************************************************/
#ifndef   _DICTIONARY_ENGINE_BASE
#define   _DICTIONARY_ENGINE_BASE

#ifdef    __cplusplus
       extern "C" {
#endif

#ifndef NOLOGS
	#include <glib/gstdio.h>
	#define eg_debug g_debug
	#define eg_warning g_warning
	#define eg_error g_error
	#define eg_critical g_critical
#else
	#define eg_debug(frm,...) while(FALSE)
	#define eg_warning(frm,...) while(FALSE)
	#define eg_error(frm,...) while(FALSE)
	#define eg_critical(frm,...) while(FALSE)
#endif

#define eg_malloc(n)	g_try_malloc0(n)
#define eg_free(n)	g_free(n); n = NULL

//______________________________________________________________________________
// *****************************************************************************
//************************************************************* HEADERS SECTION:
//------------------------------------------------------------------------------
// header with gnu library for C - most of types and functions needed it (all
// started from g* and g_*().
#include <glib.h>
//------------------------------------------------------------------------------

#define _GLOBAL_FUNCTIONS_NAME_  "engine_global_functions"
#define _GLOBAL_FUNCTIONS_ 	  engine_global_functions

//______________________________________________________________________________
// *****************************************************************************
//******************************************************* GLOBAL MACROS SECTION:
//------------------------------------------------------------------------------
#define         dict_eng_module_check(module,location)   \
                        ( (module).engine_check( (location) ) )
//------------------------------------------------------------------------------
#define         dict_eng_module_get_description(module)   \
                        ( (module).engine_description() )
//------------------------------------------------------------------------------
#define         dict_eng_module_get_version(module)   \
                        ( (module).engine_version() )
//------------------------------------------------------------------------------
#define         dict_eng_module_get_format(module)   \
                        ( (module).engine_version() )
//------------------------------------------------------------------------------
#define         dict_eng_module_create(module,location,flags)   \
        ( (module).engine_create(  (location), (flags), NULL, NULL, 0.01 ) )
//------------------------------------------------------------------------------
#define         dict_eng_module_create_ext(module,                      \
                                           location,                    \
                                           flags,                       \
                                           progress_handler,            \
                                           progress_data,               \
                                           seed                         \
                                           )  (                         \
                       (module).engine_create( (location),              \
                                               (flags),                 \
                                               (progress_handler),      \
                                               (progress_data),         \
                                               (seed)                   \
                                             ))
//------------------------------------------------------------------------------


//______________________________________________________________________________
// *****************************************************************************
//****************************************** CONCRETE DICTIONARY MACROS SECTION:
//------------------------------------------------------------------------------
#define         dict_eng_set_progress_seed(engine, signal, val) (              \
                        (engine) -> engine_set_progress_seed( (engine),        \
                                                              (signal),        \
                                                              (val)            \
                                                            ))
//------------------------------------------------------------------------------
#define         dict_eng_set_auto_free(engine, state)  \
                        (engine) -> engine_set_auto_free( (engine), (state) )
//------------------------------------------------------------------------------
#define         dict_eng_optimize(engine)  \
                        ((engine) -> engine_optimize( (engine) ))
//------------------------------------------------------------------------------
#define         dict_eng_is_optimized( engine )  \
                        ((engine) -> engine_is_optimized( (engine) ))
//------------------------------------------------------------------------------
#define         dict_eng_get_location(engine)    \
                        ((engine) -> engine_location( (engine) ))
//------------------------------------------------------------------------------
#define         dict_eng_get_last_status(engine)    \
                        ((engine) -> engine_error( (engine) ))
//------------------------------------------------------------------------------
#define         dict_eng_status_message(engine, error)     \
                        ((engine) -> engine_error_message( (error) ))
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#define         dict_eng_destroy(engine)    \
                        ((engine) -> engine_close( (engine) ))
//------------------------------------------------------------------------------
#define         dict_eng_search_word_list(engine,pattern,data)    \
                        ((engine) -> engine_search_word_list( (engine),        \
                                                              (pattern),       \
                                                              (data)           \
                                                            ))
//------------------------------------------------------------------------------
#define         dict_eng_search_word_translation(engine,word,data)    \
                        ((engine) -> engine_search_word_translation( (engine), \
                                                                     (word),   \
                                                                     (data)    \
                                                                   ))
//------------------------------------------------------------------------------
#define         dict_eng_search_word_translation_extended(engine,word)   \
                ((engine) -> engine_search_word_translation_extended((engine), \
                                                                     (pattern) \
                                                                     ))
//------------------------------------------------------------------------------
#define         dict_eng_set_callback(engine,signal,c_handler,data)     \
                        ((engine) -> engine_set_callback( (engine),     \
                                                          (signal),     \
                                                          (c_handler),  \
                                                          (data)        \
                                                        ))
//------------------------------------------------------------------------------
#define         dict_eng_add_word(engine,word,tran)    \
                        ((engine) -> engine_add_word( (engine), \
                                                       (word),   \
                                                       (tran) \
                                                     ))
//------------------------------------------------------------------------------
#define         dict_eng_remove_word(engine,word)    \
                        ((engine) -> engine_remove_word( (engine), \
                                                          (word)   \
                                                        ))
//------------------------------------------------------------------------------
#define         dict_eng_get_lang_from(engine)    \
                        ((engine) -> engine_get_lang_from( (engine) ))
//------------------------------------------------------------------------------
#define         dict_eng_get_lang_to(engine)    \
                        ((engine) -> engine_get_lang_to( (engine) ))
//------------------------------------------------------------------------------
#define         dict_eng_get_title(engine)    \
                        ((engine) -> engine_get_title( (engine) ))
//------------------------------------------------------------------------------
#define         dict_eng_get_icon_path(engine)    \
                        ((engine) -> engine_get_icon_path( (engine) ))
//------------------------------------------------------------------------------



//______________________________________________________________________________
// *****************************************************************************
//************************************ SIGNALS DEFINITION FOR CALLBACKS SECTION:
//------------------------------------------------------------------------------
#define ENGINE_WORD_LIST_SIGNAL                 "on_word_list_found"
//------------------------------------------------------------------------------
#define ENGINE_WORD_TRANSLATION_SIGNAL          "on_word_translation_found"
//------------------------------------------------------------------------------
#define ENGINE_PROGRESS_OPTIMIZING_SIGNAL       "on_progress_optimizing"
//------------------------------------------------------------------------------

//______________________________________________________________________________
// *****************************************************************************
//************************************************** DEFINITIONS OF ENUMS TYPES:
//------------------------------------------------------------------------------
typedef enum
{
        ENGINE_CREATE = 0, 
        ENGINE_NO,
        ENGINE_REFRESH 
} EngineOptimizationFlag;
//------------------------------------------------------------------------------
typedef enum
{
        ENGINE_NO_ERROR = 0,
        ENGINE_WRONG_FILE,
        ENGINE_COULDNT_READ,
	ENGINE_COULDNT_WRITE,
	ENGINE_INTERNAL_ERROR,
        ENGINE_NO_FILE,     
        ENGINE_OUT_OF_MEMORY
}
EngineStatus;
//------------------------------------------------------------------------------


//______________________________________________________________________________
// *****************************************************************************
//********************************************* DEFINITIONS OF CALLBACK'S TYPES:
//------------------------------------------------------------------------------
typedef void (*cb_word_list)(GArray* list,
                             gchar* pattern,
                             gpointer user_data,
                             EngineStatus error);
//------------------------------------------------------------------------------
typedef void (*cb_word_translation)(gchar* translation,
                                    gchar* word,
                                    gpointer user_data,
                                    EngineStatus error);
//------------------------------------------------------------------------------
typedef void (*cb_progress)(gdouble value,
                            gpointer user_data,
                            EngineStatus error);
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
struct _Engine;
typedef struct _Engine Engine;

struct _Engine
{
        // 0.1 API specification
        void            (*engine_set_auto_free)(Engine* engine,
                                                gboolean state);
        gchar*          (*engine_location)(Engine* engine);       
        gboolean        (*engine_is_optimized)(Engine* engine);
        void            (*engine_optimize)(Engine* engine);
        void            (*engine_search_word_list)(Engine* engine,
                                                   gchar* pattern,
	                                           gpointer data);
        void            (*engine_search_word_translation)(Engine* engine,
                                                          gchar* word,
	                                                  gpointer data);
        void            (*engine_close)(Engine* engine);
        // engine status
        EngineStatus    (*engine_status)(Engine* engine);
        gchar*          (*engine_error_message)(EngineStatus error);
        // callbacks
        gpointer        (*engine_set_callback)(Engine* engine, 
                                               gchar* signal, 
                                               gpointer c_handler, 
                                               gpointer user_data);
        void            (*engine_set_progress_seed)(Engine* engine,
                                                    gchar* signal,
                                                    gdouble seed);
        // 0.2 API specification's functions
        gboolean        (*engine_add_word)(Engine* engine,
                                           gchar*  word,
                                           gchar*  translation);
        gboolean        (*engine_remove_word)(Engine* engine,
                                              gchar*  word);        
        gchar*          (*engine_get_lang_from)(Engine* engine);
        gchar*          (*engine_get_lang_to)(Engine* engine);
        gchar*          (*engine_get_title)(Engine* engine);
        gchar*          (*engine_get_icon_path)(Engine* engine);

        // dictionary independent data
        gpointer engine_data;
};
//------------------------------------------------------------------------------
typedef struct  {
        gboolean (*engine_check)(gchar* location); 
        gchar*   (*engine_description)(void);     
        gchar*   (*engine_format)(void);
        gchar*   (*engine_version)(void);
        Engine*  (*engine_create)(gchar* location,
                                   EngineOptimizationFlag flags,
                                   cb_progress progress_handler,
                                   gpointer progress_data,
                                   gdouble seed);
} 
EngineModule;
//------------------------------------------------------------------------------
typedef EngineModule (*getting_additional)(void);
//------------------------------------------------------------------------------
extern EngineModule engine_global_functions(void);
//------------------------------------------------------------------------------

          
#ifdef __cplusplus
}                     
#endif
#endif /* _DICTIONARY_ENGINE_BASE */
